const { TRACKABLE } = require('enums/object_type');
const ALARM_TYPE = require('enums').ALARM_TYPE;
const { AlarmRepo } = require('../../../repository/alarm');
const { CertificationRepo } = require('../repository/certification');
const apiCreateAlarm = require('./api_utils').apiCreateAlarm;


class VehiclesEventHandler {
    static parseHexData(hexData) {
        const result = {};
        result.refNumber = hexData.slice(0, 4);
        if (hexData.length > 4) {
            result.duration = parseInt(hexData.slice(4, 8), 16);
        }
        return result;
    }

    static async clearAlarm(uid, ts, ms, reason) {
        const timestamp = (ts * 1000) + ms;
        const alarm = await AlarmRepo.findByObjUidTypeIdAndTs(uid, ALARM_TYPE.UNCERTIFIED_OPERATION.id, timestamp);

        if (alarm) {
            const postData = {
                alarm_name: ALARM_TYPE.UNCERTIFIED_OPERATION.name,
                obj_uid: alarm.obj_id,
                state: 'clear',
                obj_class: 'TO',
                additional_info: {
                    ...alarm.additional_info,
                    duration: Math.floor((timestamp - new Date(alarm.ts).getTime()) / 1000),
                    reason,
                },
                sublocation_uid: alarm.sublocation_uid,
                x: alarm.x || 0,
                y: alarm.y || 0,
                z: alarm.z || 0,
                ts,
                ms,
            };

            await apiCreateAlarm(postData);
        }
    }
    static async clearCertification(uid) {
        await CertificationRepo.deleteDriverLicenses(uid);
        await CertificationRepo.deleteVehicleCertifications(uid);
    }

    static async nodeEventHandler(nodeEvent) {
        const { type, x, y, z, obj_uid, obj_name, sublocation_uid, ts } = nodeEvent;
        const data = VehiclesEventHandler.parseHexData(nodeEvent.data);
        const additional_info = {
            text: `${obj_name} uncertified operation`,
            ...data,
        };


        if (type === '0021') {
            additional_info.clear_x = x;
            additional_info.clear_y = y;
            additional_info.clear_z = z;
            additional_info.clear_sublocation_uid = sublocation_uid;
            additional_info.reason = 'uncertified_operation_stop';
        }

        const postData = {
            alarm_name: ALARM_TYPE.UNCERTIFIED_OPERATION.name,
            obj_uid,
            state: (type === '0020') ? 'set' : 'clear',
            obj_class: 'TO',
            additional_info,
            sublocation_uid,
            x: x || 0,
            y: y || 0,
            z: z || 0,
            ts: Math.floor(ts / 1000),
            ms: ts % 1000,
        };

        await apiCreateAlarm(postData);
    }

    static async removeDriverCertification(uid, category) {
        const result = await CertificationRepo.findDriverLicenses(uid);
        if (result && category !== TRACKABLE.CATEGORY.PERSON) {
            await CertificationRepo.deleteDriverLicenses(uid);
        }
    }

    static async removeVehiclesModelAssociations(uid, category) {
        const result = await CertificationRepo.findVehCert(uid);
        if (result && category !== TRACKABLE.CATEGORY.VEHICLE) {
            await CertificationRepo.deleteVehicleCertifications(uid);
        }
    }

    static async trackableUpdateHandler(tkoUpdateNotify) {
        const { ts, ms, plist: { uid, node, category } } = tkoUpdateNotify;
        if (!node || !node.responsive) {
            const reason = !node ? 'close_association' : 'not_responsive';
            await VehiclesEventHandler.clearAlarm(uid, ts, ms, reason);
        }
        await VehiclesEventHandler.removeDriverCertification(uid, category);
        await VehiclesEventHandler.removeVehiclesModelAssociations(uid, category);
    }

    static async trackableDeleteHandler(notify) {
        const { plist: { uid } } = notify;
        await VehiclesEventHandler.clearCertification(uid);
    }

    static async nodeNAHandler(announceEvent) {
        const { ms, dev_id, ts } = announceEvent;
        await VehiclesEventHandler.clearAlarm(dev_id, ts, ms, 'reboot');
    }
}

module.exports = {
    VehiclesEventHandler,
};
