const _IsEmpty = require('lodash.isempty');
const _Unique = require('lodash.uniq');
const Service = require('./service');
const zmqUtils = require('../../../routes/utils_zmq');
const { OBJECT_TYPE } = require('enums');


function broadcastVehicleModel(vehicle_model, req, action) {
    if (!vehicle_model) return;
    zmqUtils.notify({
        req,
        object: OBJECT_TYPE.VEHICLE_MODELS,
        action,
        plist: {
            id: vehicle_model.id,
            name: vehicle_model.name,
            certification_required: !!vehicle_model.certification_required,
        },
    });
}

function broadcastDriversLicenses(drivers, req) {
    if (!Array.isArray(drivers)) return;

    if (drivers.length === 0 && req.body.obj_uid) {
        zmqUtils.notify({
            req,
            object: OBJECT_TYPE.DRIVER_LICENSES,
            action: zmqUtils.OBJECT_TYPE.TYPE_ACTION.UPDATED,
            plist: {
                driver_lic: [],
                obj_uid: req.body.obj_uid,
            },
        });
    } else if (drivers.length > 0) {
        drivers.forEach((item) => {
            const { obj_uid, licenses } = item;
            zmqUtils.notify({
                req,
                object: OBJECT_TYPE.DRIVER_LICENSES,
                action: zmqUtils.OBJECT_TYPE.TYPE_ACTION.UPDATED,
                plist: {
                    driver_lic: licenses.map(lic_item => ({
                        id: Number(lic_item.id),
                        certification_required: !!lic_item.certification_required
                    })),
                    obj_uid,
                },
            });
        });
    }
}

function broadcastVehiclesCertifications(vehicleCertification, req) {
    if (_IsEmpty(vehicleCertification)) return;

    if (Array.isArray(vehicleCertification) && vehicleCertification.length > 0) {
        vehicleCertification.forEach((vcc) => {
            const { obj_uid, id, certification_required } = vcc;
            zmqUtils.notify({
                req,
                object: OBJECT_TYPE.VEHICLE_CERTIFICATIONS,
                action: zmqUtils.OBJECT_TYPE.TYPE_ACTION.UPDATED,
                plist: {
                    id,
                    obj_uid,
                    certification_required: !!certification_required,
                },
            });
        });
    } else {
        const { obj_uid, id, certification_required } = vehicleCertification;
        zmqUtils.notify({
            req,
            object: OBJECT_TYPE.VEHICLE_CERTIFICATIONS,
            action: zmqUtils.OBJECT_TYPE.TYPE_ACTION.UPDATED,
            plist: {
                certification_required: !!certification_required,
                obj_uid,
                id,
            },
        });
    }
}

async function getVehicleModels(req, res, next) {
    try {
        const options = {
            company_uid: req.scope.subcontractorTag ? req.scope.subcontractorTag.uid : null,
            project_uid: req.projectTag && req.projectTag.uid,
        };
        const vehicle_models = await Service.getVehicleModels(options);

        res.json(vehicle_models);
    } catch (e) {
        next(e);
    }
}

async function createVehicleModel(req, res, next) {
    try {
        const options = {
            company_uid: req.scope.subcontractorTag ? req.scope.subcontractorTag.uid : null,
            project_uid: req.projectTag && req.projectTag.uid,
            name: req.data && req.data.name,
            certification_required: req.data && req.data.certification_required,
        };
        const vehicle_model = await Service.createVehicleModel(options);
        broadcastVehicleModel(vehicle_model, req, zmqUtils.OBJECT_TYPE.TYPE_ACTION.CREATED);

        res.json(vehicle_model);
    } catch (e) {
        next(e);
    }
}

async function updateVehicleModel(req, res, next) {
    try {
        const options = {
            company_uid: req.scope.subcontractorTag ? req.scope.subcontractorTag.uid : null,
            project_uid: req.projectTag && req.projectTag.uid,
            certification_required: req.data && req.data.certification_required,
            name: req.data && req.data.name,
            id: Number(req.params.id),
        };
        const { vehicle_model, drivers, vehicles } = await Service.updateVehicleModel(options);

        broadcastVehicleModel(vehicle_model, req, zmqUtils.OBJECT_TYPE.TYPE_ACTION.UPDATED);
        broadcastDriversLicenses(drivers, req);
        broadcastVehiclesCertifications(vehicles, req);
        res.json(vehicle_model);
    } catch (e) {
        next(e);
    }
}

async function deleteVehicleModel(req, res, next) {
    try {
        const options = {
            company_uid: req.scope.subcontractorTag ? req.scope.subcontractorTag.uid : null,
            project_uid: req.projectTag && req.projectTag.uid,
            id: Number(req.params.id),
        };
        const { vehicles, drivers, vehicle_model } = await Service.deleteVehicleModel(options);
        broadcastVehicleModel(vehicle_model, req, zmqUtils.OBJECT_TYPE.TYPE_ACTION.DELETED);
        broadcastDriversLicenses(drivers, req);
        broadcastVehiclesCertifications(vehicles, req);
        res.status(204).end();
    } catch (e) {
        next(e);
    }
}

async function getDriverLicenses(req, res, next) {
    try {
        const options = {
            company_uid: req.scope.subcontractorTag ? req.scope.subcontractorTag.uid : null,
            project_uid: req.projectTag && req.projectTag.uid,
        };
        const driverLicenses = await Service.getDriverLicenses(options);
        res.json(driverLicenses);
    } catch (e) {
        next(e);
    }
}

async function editDriverLicenses(req, res, next) {
    try {
        const options = {
            company_uid: req.scope.subcontractorTag ? req.scope.subcontractorTag.uid : null,
            project_uid: req.projectTag && req.projectTag.uid,
            licenses: _Unique(req.body && req.body.licenses),
            obj_uid: req.body && req.body.obj_uid,
        };
        const driverLicenses = await Service.editDriverLicenses(options);
        broadcastDriversLicenses(driverLicenses, req);
        const result = (driverLicenses.length === 0) ? { obj_uid: req.body.obj_uid, licenses: [] } : driverLicenses[0];

        res.json(result);
    } catch (e) {
        next(e);
    }
}

async function editVehicleCertificationClass(req, res, next) {
    try {
        const options = {
            company_uid: req.scope.subcontractorTag ? req.scope.subcontractorTag.uid : null,
            project_uid: req.projectTag && req.projectTag.uid,
            id: req.body && req.body.id,
            obj_uid: req.body && req.body.obj_uid,
        };
        const vehicleCertification = await Service.editVehicleCertification(options);
        broadcastVehiclesCertifications(vehicleCertification, req);
        res.json(vehicleCertification);
    } catch (e) {
        next(e);
    }
}

async function getVehiclesCertifications(req, res, next) {
    try {
        const options = {
            company_uid: req.scope.subcontractorTag ? req.scope.subcontractorTag.uid : null,
            project_uid: req.projectTag && req.projectTag.uid,
        };
        const vehicleCertifications = await Service.getVehiclesCertifications(options);
        res.json(vehicleCertifications);
    } catch (e) {
        next(e);
    }
}

module.exports = {
    getVehicleModels,
    updateVehicleModel,
    createVehicleModel,
    deleteVehicleModel,
    getDriverLicenses,
    editDriverLicenses,
    getVehiclesCertifications,
    editVehicleCertificationClass,
};
